Class: Shrine::Derivation::Opened
- Inherits:
-
Derivation::Command
- Object
- Derivation::Command
- Shrine::Derivation::Opened
- Defined in:
- lib/shrine/plugins/derivation_endpoint.rb
Instance Method Summary collapse
-
#call ⇒ Object
Returns opened Shrine::UploadedFile object pointing to the uploaded if it exists on the storage.
Instance Method Details
#call ⇒ Object
Returns opened Shrine::UploadedFile object pointing to the uploaded if it exists on the storage.
686 687 688 689 690 691 |
# File 'lib/shrine/plugins/derivation_endpoint.rb', line 686 def call uploaded_file = shrine_class.uploaded_file(storage: upload_storage, id: upload_location) uploaded_file.open(**) uploaded_file rescue Shrine::FileNotFound end |