Class: Shrine::Derivation::Retrieve
- Inherits:
-
Derivation::Command
- Object
- Derivation::Command
- Shrine::Derivation::Retrieve
- Defined in:
- lib/shrine/plugins/derivation_endpoint.rb
Instance Method Summary collapse
-
#call ⇒ Object
Returns a Shrine::UploadedFile object pointing to the uploaded derivative if it exists on the storage.
Instance Method Details
#call ⇒ Object
Returns a Shrine::UploadedFile object pointing to the uploaded derivative if it exists on the storage.
675 676 677 678 |
# File 'lib/shrine/plugins/derivation_endpoint.rb', line 675 def call uploaded_file = shrine_class.uploaded_file(storage: upload_storage, id: upload_location) uploaded_file if uploaded_file.exists? end |