Module: Shrine::Storage::S3::ClientSideEncryption
- Included in:
- Shrine::Storage::S3
- Defined in:
- lib/shrine/storage/s3.rb
Overview
Adds support for Aws::S3::Encryption::Client.
Instance Attribute Summary collapse
-
#encryption_client ⇒ Object
readonly
Returns the value of attribute encryption_client.
Instance Method Summary collapse
-
#initialize(client: nil) ⇒ Object
Save the encryption client and continue initialization with normal client.
Instance Attribute Details
#encryption_client ⇒ Object (readonly)
Returns the value of attribute encryption_client.
366 367 368 |
# File 'lib/shrine/storage/s3.rb', line 366 def encryption_client @encryption_client end |
Instance Method Details
#initialize(client: nil) ⇒ Object
Save the encryption client and continue initialization with normal client.
370 371 372 373 374 375 |
# File 'lib/shrine/storage/s3.rb', line 370 def initialize(client: nil, **) return super unless client.class.name.start_with?("Aws::S3::Encryption") super(client: client.client, **) @encryption_client = client end |